@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* alias.h: alias substitution */
-/* (C) 2007-2009 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ extern const wchar_t *get_alias_value(const wchar_t *aliasname)
     __attribute__((nonnull,pure));
 extern void destroy_aliaslist(struct aliaslist_T *list);
 extern _Bool substitute_alias(
-	struct xwcsbuf_T *restrict buf, size_t i, size_t len,
+	struct xwcsbuf_T *restrict buf, size_t i,
 	struct aliaslist_T **restrict list, substaliasflags_T flags)
     __attribute__((nonnull));
 extern _Bool print_alias_if_defined(
