@@ -24,6 +24,9 @@
 #if HAVE_GETGRENT
 # include <grp.h>
 #endif
+#if HAVE_GETTEXT
+# include <libintl.h>
+#endif
 #if HAVE_GETHOSTENT
 # include <netdb.h>
 #endif
@@ -39,9 +42,6 @@
 #include <wchar.h>
 #include <wctype.h>
 #include <sys/stat.h>
-#if HAVE_GETTEXT
-# include <libintl.h>
-#endif
 #include "../builtin.h"
 #include "../exec.h"
 #include "../expand.h"
@@ -668,7 +668,7 @@ bool call_standard_completion_function(void)
 void set_completion_variables(void)
 {
     set_array(L VAR_WORDS, ctxt->pwordc, pldup(ctxt->pwords, copyaswcs),
-	    SCOPE_LOCAL);
+	    SCOPE_LOCAL, false);
     set_variable(L VAR_TARGETWORD, xwcsdup(ctxt->src), SCOPE_LOCAL, false);
     set_variable(L VAR_IFS, xwcsdup(DEFAULT_IFS), SCOPE_LOCAL, false);
 }
