@@ -19,6 +19,9 @@
 #include "common.h"
 #include <assert.h>
 #include <errno.h>
+#if HAVE_GETTEXT
+# include <libintl.h>
+#endif
 #include <limits.h>
 #include <stdarg.h>
 #include <stdbool.h>
@@ -28,9 +31,6 @@
 #include <unistd.h>
 #include <wchar.h>
 #include <wctype.h>
-#if HAVE_GETTEXT
-# include <libintl.h>
-#endif
 #if YASH_ENABLE_ALIAS
 # include "alias.h"
 #endif
@@ -546,8 +546,8 @@ parseresult_T read_and_parse(
     cindex = 0;
     wb_init(&cbuf);
 
-    if (info->intrinput) {
-	struct input_interactive_info *intrinfo = info->inputinfo;
+    if (info->interactive) {
+	struct input_interactive_info_T *intrinfo = info->inputinfo;
 	intrinfo->prompttype = 1;
     }
 
@@ -628,7 +628,7 @@ void serror(const char *restrict format, ...)
  * INPUT_INTERRUPTED. */
 inputresult_T read_more_input(void)
 {
-    if (cerror && cinfo->intrinput)
+    if (cerror && cinfo->interactive)
 	return INPUT_INTERRUPTED;
     if (cinfo->lastinputresult == INPUT_OK) {
 	size_t savelength = cbuf.length;
@@ -858,6 +858,9 @@ and_or_T *parse_command_list(void)
 	} else if (cbuf.contents[cindex] == L'\n') {
 	    next_line();
 	    break;
+	} else if (cbuf.contents[cindex] == L')') {
+	    serror(Ngt(get_errmsg_unexpected_token(L")")), L")");
+	    break;
 	} else if (!separator) {
 	    serror(Ngt("`;' or `&' is missing"));
 	    break;
@@ -900,7 +903,7 @@ and_or_T *parse_compound_list(void)
     /* To parse a command after another, there must be "&", ";" or one or more
      * newlines as the separator. */
 
-    if (!cinfo->intrinput)
+    if (!cinfo->interactive)
 	cerror = false;
     while (!cerror) {
 	separator |= skip_to_next_token();
