@@ -21,21 +21,23 @@
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
+#if HAVE_GETTEXT
+# include <libintl.h>
+#endif
 #include <limits.h>
+#if YASH_ENABLE_SOCKET
+# include <netdb.h>
+#endif
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/select.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#if HAVE_GETTEXT
-# include <libintl.h>
-#endif
 #if YASH_ENABLE_SOCKET
-# include <netdb.h>
 # include <sys/socket.h>
 #endif
+#include <sys/stat.h>
+#include <unistd.h>
 #include "exec.h"
 #include "expand.h"
 #include "input.h"
@@ -294,7 +296,8 @@ struct savefd_T {
 
 static char *expand_redir_filename(const struct wordunit_T *filename)
     __attribute__((malloc,warn_unused_result));
-static void save_fd(int oldfd, savefd_T **save);
+static void save_fd(int oldfd, savefd_T **save)
+    __attribute__((nonnull));
 static int open_file(const char *path, int oflag)
     __attribute__((nonnull));
 #if YASH_ENABLE_SOCKET
@@ -318,8 +321,7 @@ static int open_process_redirection(const embedcmd_T *command, redirtype_T type)
  * Returns true iff successful. */
 bool open_redirections(const redir_T *r, savefd_T **save)
 {
-    if (save != NULL)
-	*save = NULL;
+    *save = NULL;
 
     while (r != NULL) {
 	if (r->rd_fd < 0 || is_shellfd(r->rd_fd)) {
@@ -440,9 +442,9 @@ openwithflags:
 char *expand_redir_filename(const struct wordunit_T *filename)
 {
     if (is_interactive) {
-	return expand_single_with_glob(filename, tt_single);
+	return expand_single_with_glob(filename, TT_SINGLE);
     } else {
-	wchar_t *result = expand_single(filename, tt_single);
+	wchar_t *result = expand_single(filename, TT_SINGLE);
 	if (result == NULL)
 	    return NULL;
 	char *mbsresult = realloc_wcstombs(unescapefree(result));
@@ -456,8 +458,6 @@ char *expand_redir_filename(const struct wordunit_T *filename)
 void save_fd(int fd, savefd_T **save)
 {
     assert(fd >= 0);
-    if (save == NULL)
-	return;
 
     int copyfd = copy_as_shellfd(fd);
     if (copyfd < 0 && errno != EBADF) {
