@@ -24,6 +24,7 @@
 #include <signal.h>
 #include <stdbool.h>
 #include <stdint.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/select.h>
@@ -237,8 +238,6 @@ static volatile sig_atomic_t any_signal_received = false;
 
 /* the signal for which trap is currently executed */
 static int handled_signal = -1;
-/* set to true when the EXIT trap is executed */
-static bool exit_handled = false;
 
 /* flags to indicate a signal is caught. */
 static volatile sig_atomic_t signal_received[MAXSIGIDX];
@@ -683,7 +682,8 @@ int handle_traps(void)
 
     int signum = 0;
     bool save_sigint_received = sigint_received;
-    struct parsestate_T *state = NULL;
+    struct parsestate_T *parsestate = NULL;
+    struct execstate_T *execstate = NULL;
     savelaststatus = laststatus;
 
     sigint_received = false;
@@ -701,8 +701,10 @@ int handle_traps(void)
 #if YASH_ENABLE_LINEEDIT
 		    le_suspend_readline();
 #endif
-		    if (state == NULL)
-			state = save_parse_state();
+		    if (parsestate == NULL) {
+			parsestate = save_parse_state();
+			execstate = save_execstate();
+		    }
 		    signum = handled_signal = s->no;
 		    command = xwcsdup(command);
 		    exec_wcs(command, "trap", false);
@@ -723,8 +725,10 @@ int handle_traps(void)
 #if YASH_ENABLE_LINEEDIT
 		    le_suspend_readline();
 #endif
-		    if (state == NULL)
-			state = save_parse_state();
+		    if (parsestate == NULL) {
+			parsestate = save_parse_state();
+			execstate = save_execstate();
+		    }
 		    signum = handled_signal = sigrtmin + i;
 		    command = xwcsdup(command);
 		    exec_wcs(command, "trap", false);
@@ -739,8 +743,10 @@ int handle_traps(void)
     sigint_received |= save_sigint_received;
     savelaststatus = -1;
     handled_signal = -1;
-    if (state != NULL)
-	restore_parse_state(state);
+    if (parsestate != NULL) {
+	restore_parse_state(parsestate);
+	restore_execstate(execstate);
+    }
 #if YASH_ENABLE_LINEEDIT
     if (shopt_notifyle && (le_state & LE_STATE_SUSPENDED))
 	print_job_status_all();
@@ -750,15 +756,20 @@ int handle_traps(void)
     return signum;
 }
 
-/* Executes the EXIT trap if any. */
+/* Executes the EXIT trap if any.
+ * This function calls `reset_execstate' without saving `execstate'. */
 void execute_exit_trap(void)
 {
     wchar_t *command = trap_command[sigindex(0)];
     if (command != NULL) {
+#ifndef NDEBUG
+	static bool exit_handled = false;
 	assert(!exit_handled);
 	exit_handled = true;
+#endif
 	savelaststatus = laststatus;
 	command = xwcsdup(command);
+	reset_execstate();
 	exec_wcs(command, "EXIT trap", false);
 	free(command);
 	savelaststatus = -1;
@@ -887,6 +898,9 @@ bool is_ignored(int signum)
 {
     assert(!is_interactive_now);
 
+    if (signum == 0)
+	return false;
+
     if (doing_job_control_now && signum == SIGTSTP)
 	return sigismember(&ignored_signals, signum);
 
@@ -896,6 +910,12 @@ bool is_ignored(int signum)
 	&& action.sa_handler == SIG_IGN;
 }
 
+/* Clears the EXIT trap. */
+void clear_exit_trap(void)
+{
+    set_trap(0, NULL);
+}
+
 /* Clears all traps except that are set to SIG_IGN. */
 void clear_traps(void)
 {
@@ -933,6 +953,14 @@ _Bool is_interrupted(void)
     return is_interactive_now && sigint_received;
 }
 
+/* Sets `laststatus' to (SIGINT + TERMSIGOFFSET) if the shell has been
+ * interrupted. */
+void set_laststatus_if_interrupted(void)
+{
+    if (is_interrupted())
+	laststatus = SIGINT + TERMSIGOFFSET;
+}
+
 /* Sets the `sigint_received' flag. */
 void set_interrupted(void)
 {
