@@ -20,6 +20,9 @@
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
+#if HAVE_GETTEXT
+# include <libintl.h>
+#endif
 #include <locale.h>
 #include <stdbool.h>
 #include <stdio.h>
@@ -27,9 +30,6 @@
 #include <string.h>
 #include <unistd.h>
 #include <wchar.h>
-#if HAVE_GETTEXT
-# include <libintl.h>
-#endif
 #if YASH_ENABLE_ALIAS
 # include "alias.h"
 #endif
@@ -55,14 +55,14 @@
 
 extern int main(int argc, char **argv)
     __attribute__((nonnull));
-static struct input_file_info *new_input_file_info(int fd, size_t bufsize)
+static struct input_file_info_T *new_input_file_info(int fd, size_t bufsize)
     __attribute__((malloc,warn_unused_result));
 static void execute_profile(const wchar_t *profile);
 static void execute_rcfile(const wchar_t *rcfile);
 static void print_help(void);
 static void print_version(void);
 
-static void parse_and_exec(struct parseinfo_T *pinfo, _Bool finally_exit)
+static void parse_and_exec(struct parseinfo_T *pinfo, bool finally_exit)
     __attribute__((nonnull(1)));
 static bool input_is_interactive_terminal(const parseinfo_T *pinfo)
     __attribute__((nonnull));
@@ -74,14 +74,14 @@ pid_t shell_pid;
  * In a job-controlled subshell, this value is set to the subshell's pgid. */
 pid_t shell_pgid;
 
-/* If this flag is true when the "exit" builtin is invoked, the -f option is
+/* If this flag is true when the "exit" built-in is invoked, the -f option is
  * assumed to be specified. */
 static bool forceexit;
 /* The next value of `forceexit'. */
 bool nextforceexit;
 
-/* The `input_file_info' structure for reading from the standard input. */
-struct input_file_info *stdin_input_file_info;
+/* The `input_file_info_T' structure for reading from the standard input. */
+struct input_file_info_T *stdin_input_file_info;
 
 
 /* The "main" function. The execution of the shell starts here. */
@@ -116,10 +116,10 @@ int main(int argc, char **argv)
     wargv[argc] = NULL;
 
     /* parse argv[0] */
-    yash_program_invocation_name = wargv[0] ? wargv[0] : L"";
+    yash_program_invocation_name = wargv[0];
     yash_program_invocation_short_name
 	= wcsrchr(yash_program_invocation_name, L'/');
-    if (yash_program_invocation_short_name)
+    if (yash_program_invocation_short_name != NULL)
 	yash_program_invocation_short_name++;
     else
 	yash_program_invocation_short_name = yash_program_invocation_name;
@@ -208,7 +208,7 @@ int main(int argc, char **argv)
     }
 
 #if YASH_ENABLE_LINEEDIT
-    /* enable lineedit if interactive and connected to a terminal */
+    /* enable line editing if interactive and connected to a terminal */
     if (!options.lineedit_set && shopt_lineedit == SHOPT_NOLINEEDIT)
 	if (is_interactive && isatty(STDIN_FILENO) && isatty(STDERR_FILENO))
 	    set_lineedit_option(SHOPT_VI);
@@ -223,7 +223,7 @@ int main(int argc, char **argv)
 	    ensure_foreground();
     }
     set_signals();
-    set_positional_parameters(wargv + xoptind);
+    set_positional_parameters(&wargv[xoptind]);
 
     if (is_login_shell && !posixly_correct && !options.noprofile)
 	if (getuid() == geteuid() && getgid() == getegid())
@@ -240,13 +240,14 @@ int main(int argc, char **argv)
     assert(false);
 }
 
-struct input_file_info *new_input_file_info(int fd, size_t bufsize)
+struct input_file_info_T *new_input_file_info(int fd, size_t bufsize)
 {
-    struct input_file_info *info = xmalloc(sizeof *info + bufsize);
+    struct input_file_info_T *info
+	= xmallocs(sizeof *info, bufsize, sizeof *info->buf);
     info->fd = fd;
     info->bufpos = info->bufmax = 0;
     info->bufsize = bufsize;
-    memset(&info->state, 0, sizeof info->state);
+    memset(&info->state, 0, sizeof info->state);  // initial shift state
     return info;
 }
 
@@ -422,7 +423,7 @@ void print_version(void)
  * If there are no commands in `code', `laststatus' is set to zero. */
 void exec_wcs(const wchar_t *code, const char *name, bool finally_exit)
 {
-    struct input_wcs_info iinfo = {
+    struct input_wcs_info_T iinfo = {
 	.src = code,
     };
     struct parseinfo_T pinfo = {
@@ -435,19 +436,20 @@ void exec_wcs(const wchar_t *code, const char *name, bool finally_exit)
 	.lineno = 1,
 	.input = input_wcs,
 	.inputinfo = &iinfo,
-	.intrinput = false,
+	.interactive = false,
     };
 
     parse_and_exec(&pinfo, finally_exit);
 }
 
 /* Parses the input from the specified file descriptor and executes commands.
- * The file descriptor must be either STDIN_FILENO or a shell FD.
+ * The file descriptor must be either STDIN_FILENO or a shell FD. If the file
+ * descriptor is STDIN_FILENO, `finally_exit' must be true.
  * If `name' is non-NULL, it is printed in an error message on syntax error.
- * If `intrinput' is true, the input is considered interactive.
+ * If `interactive' is true, the input is considered interactive.
  * If there are no commands in the input, `laststatus' is set to zero. */
 void exec_input(int fd, const char *name,
-	bool intrinput, bool enable_alias, bool finally_exit)
+	bool interactive, bool enable_alias, bool finally_exit)
 {
     struct parseinfo_T pinfo = {
 	.print_errmsg = true,
@@ -457,26 +459,32 @@ void exec_input(int fd, const char *name,
 #endif
 	.filename = name,
 	.lineno = 1,
-	.intrinput = intrinput,
+	.interactive = interactive,
     };
-    struct input_interactive_info intrinfo;
+    struct input_interactive_info_T intrinfo;
+    struct input_file_info_T *inputinfo;
 
-    struct input_file_info *info =
-	(fd == STDIN_FILENO)
-	? stdin_input_file_info
-	: new_input_file_info(fd, BUFSIZ);
+    if (fd == STDIN_FILENO)
+	inputinfo = stdin_input_file_info;
+    else
+	inputinfo = new_input_file_info(fd, BUFSIZ);
 
-    if (intrinput) {
-	intrinfo.fileinfo = info;
+    if (interactive) {
+	intrinfo.fileinfo = inputinfo;
 	intrinfo.prompttype = 1;
+#if YASH_ENABLE_LINEEDIT
+	intrinfo.linebuffer = NULL;
+#endif
 	pinfo.input = input_interactive;
 	pinfo.inputinfo = &intrinfo;
     } else {
 	pinfo.input = input_file;
-	pinfo.inputinfo = info;
+	pinfo.inputinfo = inputinfo;
     }
     parse_and_exec(&pinfo, finally_exit);
-    free(info);
+
+    assert(inputinfo != stdin_input_file_info);
+    free(inputinfo);
 
 #if !YASH_ENABLE_ALIAS
     (void) enable_alias;  // suppress compiler warning
@@ -487,34 +495,29 @@ void exec_input(int fd, const char *name,
  * If no commands were executed, `laststatus' is set to zero. */
 void parse_and_exec(parseinfo_T *pinfo, bool finally_exit)
 {
-    struct execinfo *INIT(ei);
-    if (!finally_exit)
-	ei = save_execinfo();
-
     bool executed = false;
 
+    if (pinfo->interactive)
+	disable_return();
+
     for (;;) {
-	if (pinfo->intrinput) {
+	if (pinfo->interactive) {
+	    set_laststatus_if_interrupted();
 	    forceexit = nextforceexit;
 	    nextforceexit = false;
-	    if (!is_interrupted() && return_pending()) {
-		xerror(0, Ngt("return: not in a function or sourced file"));
-		laststatus = Exit_FAILURE;
-	    }
-	    reset_execinfo();
 	    pinfo->lineno = 1;
 	} else {
-	    if (return_pending())
+	    if (need_break())
 		goto out;
 	}
 
 	and_or_T *commands;
 	switch (read_and_parse(pinfo, &commands)) {
 	    case PR_OK:
-		if (commands) {
+		if (commands != NULL) {
 		    if (shopt_exec || is_interactive) {
 			exec_and_or_lists(commands,
-				finally_exit && !pinfo->intrinput &&
+				finally_exit && !pinfo->interactive &&
 				pinfo->lastinputresult == INPUT_EOF);
 			executed = true;
 		    }
@@ -531,13 +534,14 @@ void parse_and_exec(parseinfo_T *pinfo, bool finally_exit)
 		if (finally_exit) {
 		    wchar_t argv0[] = L"EOF";
 		    exit_builtin(1, (void *[]) { argv0, NULL });
-		} else
+		} else {
 		    goto out;
+		}
 		break;
 	    case PR_SYNTAX_ERROR:
 		if (!is_interactive_now)
 		    exit_shell_with_status(Exit_SYNERROR);
-		if (!pinfo->intrinput) {
+		if (!pinfo->interactive) {
 		    laststatus = Exit_SYNERROR;
 		    goto out;
 		}
@@ -548,17 +552,17 @@ void parse_and_exec(parseinfo_T *pinfo, bool finally_exit)
 	}
     }
 out:
+    cancel_return();
     if (finally_exit)
 	exit_shell();
-    load_execinfo(ei);
 }
 
 bool input_is_interactive_terminal(const parseinfo_T *pinfo)
 {
-    if (!pinfo->intrinput)
+    if (!pinfo->interactive)
 	return false;
 
-    struct input_interactive_info *ir = pinfo->inputinfo;
+    struct input_interactive_info_T *ir = pinfo->inputinfo;
     return isatty(ir->fileinfo->fd);
 }
 
